"""
fobis.cli — FoBiS.py CLI sub-package.

This package owns the Typer application and all subcommand definitions.
Symbols re-exported here preserve the public API that the rest of
FoBiS.py (and user code) previously imported from ``fobis.cli_parser``.
"""

# Copyright (C) 2015  Stefano Zaghi
#
# This file is part of FoBiS.py.
#
# FoBiS.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FoBiS.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FoBiS.py. If not, see <http://www.gnu.org/licenses/>.

# Import command modules to register their @app.command decorators
from . import build, clean, doctests, fetch, install, rule  # noqa: F401
from ._app import _normalize_args, app
from ._constants import (
    __compiler_supported__,
    __extensions_inc__,
    __extensions_modern__,
    __extensions_old__,
    __extensions_parsed__,
)

__all__ = [
    "__compiler_supported__",
    "__extensions_inc__",
    "__extensions_modern__",
    "__extensions_old__",
    "__extensions_parsed__",
    "_normalize_args",
    "app",
]



"""fetch.py — FoBiS.py ``fetch`` subcommand."""

from typing import Annotated

import typer

from ._app import _ns, app
from ._options import *


@app.command("fetch")
def cmd_fetch(
    ctx: typer.Context,
    # fobos group
    fobos: FobosOpt = None,
    fobos_case_insensitive: FciOpt = False,
    mode: ModeOpt = None,
    lmodes: LmodesOpt = False,
    print_fobos_template: PrintFobosTemplateOpt = False,
    # fetch-specific options
    deps_dir: Annotated[
        str | None,
        typer.Option(
            "--deps-dir",
            help="Directory for storing fetched dependencies [default: .fobis_deps or fobos [dependencies] setting]",
        ),
    ] = None,
    update: Annotated[
        bool, typer.Option("--update", help="Update already-fetched dependencies (git fetch + re-checkout)")
    ] = False,
    no_build: Annotated[bool, typer.Option("--no-build", help="Only fetch dependencies, do not build them")] = False,
    # fancy group
    force_compile: ForceCompileOpt = False,
    colors: ColorsOpt = False,
    log: LogOpt = False,
    graph: GraphOpt = False,
    quiet: QuietOpt = False,
    verbose: VerboseOpt = False,
    jobs: JobsOpt = 1,
    makefile: MakefileOpt = None,
    json_output: JsonOpt = False,
):
    """Fetch and build GitHub-hosted Fortran dependencies listed in the fobos [dependencies] section."""
    ctx.ensure_object(dict)
    ctx.obj["cliargs"] = _ns(
        which="fetch",
        fobos=fobos,
        fobos_case_insensitive=fobos_case_insensitive,
        mode=mode,
        lmodes=lmodes,
        print_fobos_template=print_fobos_template,
        deps_dir=deps_dir,
        update=update,
        no_build=no_build,
        force_compile=force_compile,
        colors=colors,
        log=log,
        graph=graph,
        quiet=quiet,
        verbose=verbose,
        jobs=jobs,
        makefile=makefile,
        json_output=json_output,
    )



"""
_app.py — Typer application instance, arg normaliser, and namespace factory.
"""

# Copyright (C) 2015  Stefano Zaghi
#
# This file is part of FoBiS.py.
#
# FoBiS.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FoBiS.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FoBiS.py. If not, see <http://www.gnu.org/licenses/>.

import argparse
import re
from typing import Annotated

import typer

# ---------------------------------------------------------------------------
# Argument normaliser — preserves backward compat with argparse-style options
# ---------------------------------------------------------------------------
_MULTI_CHAR_OPT = re.compile(r"^-[A-Za-z][A-Za-z0-9_-]+$")


def _normalize_args(args):
    """
    Normalise FoBiS legacy single-dash long options for Click/Typer.

    Rules applied to each token:
    - Single-dash multi-char option (-compiler, -mode, -get_output_name)
      → double-dash with underscores turned to hyphens (--compiler, --mode, --get-output-name)
    - Double-dash option with underscores (--build_dir, --cflags_heritage)
      → double-dash with hyphens (--build-dir, --cflags-heritage)
    - Single-char short options (-f, -m, -q), values, and negative numbers
      are left unchanged.
    """
    result = []
    for arg in args:
        if _MULTI_CHAR_OPT.match(arg):
            result.append("--" + arg[1:].replace("_", "-"))
        elif arg.startswith("--") and len(arg) > 2:
            if "=" in arg:
                opt, val = arg[2:].split("=", 1)
                result.append("--" + opt.replace("_", "-") + "=" + val)
            else:
                result.append("--" + arg[2:].replace("_", "-"))
        else:
            result.append(arg)
    return result


# ---------------------------------------------------------------------------
# Typer application
# ---------------------------------------------------------------------------
app = typer.Typer(
    name="FoBiS.py",
    help="a Fortran Building System",
    no_args_is_help=True,
    add_completion=True,
    rich_markup_mode=None,
)


def _version_callback(value: bool):
    if value:
        from .. import __version__
        from ..FoBiSConfig import __appname__

        typer.echo(f"{__appname__} {__version__}")
        raise typer.Exit()


@app.callback()
def _app_callback(
    ctx: typer.Context,
    version: Annotated[
        bool,
        typer.Option(
            "--version",
            "-v",
            help="Show version and exit.",
            callback=_version_callback,
            is_eager=True,
        ),
    ] = False,
):
    ctx.ensure_object(dict)


# ---------------------------------------------------------------------------
# Namespace factory
# ---------------------------------------------------------------------------


def _ns(**kwargs) -> argparse.Namespace:
    """Build an argparse.Namespace — preserves the duck-type expected by all downstream code."""
    return argparse.Namespace(**kwargs)



"""
_constants.py — file extension and compiler constants for FoBiS.py CLI.

These constants are re-exported from the top-level cli_parser module for
backward compatibility.
"""

# Copyright (C) 2015  Stefano Zaghi
#
# This file is part of FoBiS.py.
#
# FoBiS.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FoBiS.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FoBiS.py. If not, see <http://www.gnu.org/licenses/>.

__extensions_inc__ = [".inc", ".INC", ".h", ".H"]
__extensions_old__ = [".f", ".F", ".for", ".FOR", ".fpp", ".FPP", ".fortran", ".f77", ".F77"]
__extensions_modern__ = [".f90", ".F90", ".f95", ".F95", ".f03", ".F03", ".f08", ".F08", ".f2k", ".F2K"]
__extensions_parsed__ = __extensions_inc__ + __extensions_old__ + __extensions_modern__
__compiler_supported__ = (
    "gnu",
    "intel",
    "intel_nextgen",
    "g95",
    "opencoarrays-gnu",
    "pgi",
    "ibm",
    "nag",
    "nvfortran",
    "amd",
    "custom",
)



"""
_options.py — Shared Annotated option type aliases for FoBiS.py CLI commands.

Each alias captures the Typer Option metadata (flag names, help text,
autocompletion).  Command functions use these as parameter type annotations;
the default value is still declared at the call site so Typer can inspect it.
"""

# Copyright (C) 2015  Stefano Zaghi
#
# This file is part of FoBiS.py.
#
# FoBiS.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FoBiS.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FoBiS.py. If not, see <http://www.gnu.org/licenses/>.

from typing import Annotated

import typer

from ._completions import (
    _complete_compiler,
    _complete_extensions,
    _complete_fobos_mode,
    _complete_mklib,
)

# ---------------------------------------------------------------------------
# Fobos group  (shared by all 6 commands)
# ---------------------------------------------------------------------------

FobosOpt = Annotated[
    str | None, typer.Option("--fobos", "-f", help='Specify a "fobos" file named differently from "fobos"')
]
FciOpt = Annotated[bool, typer.Option("--fci", help="Assume fobos inputs as case insensitive")]
ModeOpt = Annotated[
    str | None,
    typer.Option("--mode", help="Select a mode defined into a fobos file", autocompletion=_complete_fobos_mode),
]
LmodesOpt = Annotated[bool, typer.Option("--lmodes", help="List the modes defined into a fobos file")]
PrintFobosTemplateOpt = Annotated[
    bool, typer.Option("--print-fobos-template", help="Print a comprehensive fobos template")
]

# ---------------------------------------------------------------------------
# Compiler group  (build, clean, rule, doctests)
# ---------------------------------------------------------------------------

CompilerOpt = Annotated[
    str,
    typer.Option("--compiler", help="Compiler used (case insensitive, default gnu)", autocompletion=_complete_compiler),
]
FcOpt = Annotated[str | None, typer.Option("--fc", help="Fortran compiler statement (for --compiler custom)")]
CflagsOpt = Annotated[str | None, typer.Option("--cflags", help="Compile flags")]
LflagsOpt = Annotated[str | None, typer.Option("--lflags", help="Link flags")]
ModswOpt = Annotated[str | None, typer.Option("--modsw", help="Module search path switch (for --compiler custom)")]
MpiOpt = Annotated[bool, typer.Option("--mpi", help="Use MPI enabled version of compiler")]
OpenmpOpt = Annotated[bool, typer.Option("--openmp", help="Use OpenMP pragmas")]
OpenmpOffloadOpt = Annotated[bool, typer.Option("--openmp-offload", help="Use OpenMP Offload pragmas")]
CoarrayOpt = Annotated[bool, typer.Option("--coarray", help="Use coarrays")]
CoverageOpt = Annotated[bool, typer.Option("--coverage", help="Instrument for coverage analysis")]
ProfileOpt = Annotated[bool, typer.Option("--profile", help="Instrument for profiling analysis")]
MklibOpt = Annotated[
    str | None,
    typer.Option("--mklib", help="Build library instead of program (static|shared)", autocompletion=_complete_mklib),
]
ArOpt = Annotated[str, typer.Option("--ar", help="Archiver executable for static libraries [default: ar]")]
ArflagsOpt = Annotated[str, typer.Option("--arflags", help="Archiver flags for static libraries [default: -rcs]")]
RanlibOpt = Annotated[str, typer.Option("--ranlib", help="Ranlib executable; empty string to skip [default: ranlib]")]
CflagsHeritageOpt = Annotated[
    bool,
    typer.Option("--cflags-heritage", "--ch", help="Store cflags for heritage checking; re-compile all if they change"),
]
TrackBuildOpt = Annotated[bool, typer.Option("--track-build", "--tb", help="Store build info for the install command")]

# ---------------------------------------------------------------------------
# Directories group  (build, clean, rule, doctests)
# ---------------------------------------------------------------------------

SrcOpt = Annotated[list[str] | None, typer.Option("--src", "-s", help="Root-directory of source files [default: ./]")]
BuildDirOpt = Annotated[
    str, typer.Option("--build-dir", "--dbld", help="Directory containing built objects [default: ./]")
]
ObjDirOpt = Annotated[
    str, typer.Option("--obj-dir", "--dobj", help="Directory containing compiled objects [default: ./obj/]")
]
ModDirOpt = Annotated[
    str, typer.Option("--mod-dir", "--dmod", help="Directory containing .mod files [default: ./mod/]")
]
LibDirOpt = Annotated[list[str] | None, typer.Option("--lib-dir", "--dlib", help="Directories searched for libraries")]
IncludeListOpt = Annotated[
    list[str] | None, typer.Option("--include", "-i", help="Directories for searching included files")
]
ExcludeDirsOpt = Annotated[
    list[str] | None,
    typer.Option("--exclude-dirs", "--ed", help="Exclude directories from the building process"),
]
DisableRecursiveSearchOpt = Annotated[
    bool, typer.Option("--disable-recursive-search", "--drs", help="Disable recursive search inside directories")
]

# ---------------------------------------------------------------------------
# Files group  (build, clean, rule, doctests)
# ---------------------------------------------------------------------------

TargetOpt = Annotated[
    str | None, typer.Option("--target", "-t", help="Specify a target file [default: all programs found]")
]
OutputOpt = Annotated[str | None, typer.Option("--output", "-o", help="Output file name (used with --target)")]
ExcludeOpt = Annotated[
    list[str] | None, typer.Option("--exclude", "-e", help="Exclude files from the building process")
]
LibsOpt = Annotated[list[str] | None, typer.Option("--libs", help="External libraries with full paths")]
VlibsOpt = Annotated[list[str] | None, typer.Option("--vlibs", help="Volatile external libraries with full paths")]
ExtLibsOpt = Annotated[list[str] | None, typer.Option("--ext-libs", help="External libraries in compiler path")]
ExtVlibsOpt = Annotated[
    list[str] | None, typer.Option("--ext-vlibs", help="Volatile external libraries in compiler path")
]
DependonOpt = Annotated[
    list[str] | None,
    typer.Option("--dependon", help="Interdependent external fobos files for interdependent building"),
]
IncOpt = Annotated[
    list[str] | None,
    typer.Option("--inc", help="Extensions for include files", autocompletion=_complete_extensions),
]
ExtensionsOpt = Annotated[
    list[str] | None,
    typer.Option("--extensions", help="Extensions of parsed files", autocompletion=_complete_extensions),
]
BuildAllOpt = Annotated[bool, typer.Option("--build-all", help="Build all sources parsed")]

# ---------------------------------------------------------------------------
# Preprocessor group  (build, rule, doctests)
# ---------------------------------------------------------------------------

PreprocessorOpt = Annotated[
    str | None,
    typer.Option("--preprocessor", help="Preprocessor name for pre-processing sources (e.g. PreForM.py)"),
]
PreprocOpt = Annotated[str | None, typer.Option("--preproc", help="Preprocessor flags for the main compiler")]
PreprocessorArgsOpt = Annotated[str, typer.Option("--preprocessor-args", "--app", help="Preprocessor-specific flags")]
PreprocessorNoOOpt = Annotated[
    bool,
    typer.Option("--preprocessor-no-o", "--npp", help="Do not add -o in front of output in preprocessor command"),
]
PreprocessorDirOpt = Annotated[
    str | None, typer.Option("--preprocessor-dir", "--dpp", help="Directory for preprocessed files")
]
PreprocessorExtOpt = Annotated[
    list[str] | None, typer.Option("--preprocessor-ext", "--epp", help="File extensions to preprocess")
]

# ---------------------------------------------------------------------------
# Fancy group  (shared by all 6 commands)
# ---------------------------------------------------------------------------

ForceCompileOpt = Annotated[bool, typer.Option("--force-compile", help="Force to (re-)compile all")]
ColorsOpt = Annotated[bool, typer.Option("--colors", help="Activate colors in shell prints")]
LogOpt = Annotated[bool, typer.Option("--log", "-l", help="Activate log file creation")]
GraphOpt = Annotated[bool, typer.Option("--graph", help="Generate a dependencies graph via graphviz")]
QuietOpt = Annotated[bool, typer.Option("--quiet", "-q", help="Less verbose output")]
VerboseOpt = Annotated[bool, typer.Option("--verbose", help="Extremely verbose output for debugging")]
JobsOpt = Annotated[int, typer.Option("--jobs", "-j", help="Number of concurrent compilation jobs [default: 1]")]
MakefileOpt = Annotated[
    str | None, typer.Option("--makefile", "-m", help="Generate a GNU Makefile for building the project")
]

# ---------------------------------------------------------------------------
# rule-specific options
# ---------------------------------------------------------------------------

ExecuteOpt = Annotated[
    str | None, typer.Option("--execute", "--ex", help="Specify a rule (defined in fobos) to execute")
]
ListRulesOpt = Annotated[bool, typer.Option("--list", "--ls", help="List the rules defined in a fobos file")]
GetOpt = Annotated[str | None, typer.Option("--get", help="Get option value defined in fobos (e.g. --get build_dir)")]
GetOutputNameOpt = Annotated[
    bool, typer.Option("--get-output-name", help="Get the final output name from fobos options")
]
FordOpt = Annotated[str | None, typer.Option("--ford", help="Build docs with Ford tool (specify project-file.md)")]
GcovAnalyzerOpt = Annotated[
    list[str] | None,
    typer.Option("--gcov-analyzer", help="Analyse .gcov files; args: REPORTS_DIR [SUMMARY_FILE]"),
]
IsAsciiKindSupportedOpt = Annotated[
    bool, typer.Option("--is-ascii-kind-supported", help="Check if compiler supports ASCII kind")
]
IsUcs4KindSupportedOpt = Annotated[
    bool, typer.Option("--is-ucs4-kind-supported", help="Check if compiler supports UCS4 kind")
]
IsFloat128KindSupportedOpt = Annotated[
    bool, typer.Option("--is-float128-kind-supported", help="Check if compiler supports float128 kind")
]

# ---------------------------------------------------------------------------
# doctests-specific options
# ---------------------------------------------------------------------------

KeepVolatileDoctestsOpt = Annotated[
    bool, typer.Option("--keep-volatile-doctests", help="Keep the volatile doctests programs")
]
ExcludeFromDoctestsOpt = Annotated[
    list[str] | None, typer.Option("--exclude-from-doctests", help="Exclude files from the doctests process")
]

# ---------------------------------------------------------------------------
# Output format option  (build, clean, fetch)
# ---------------------------------------------------------------------------

JsonOpt = Annotated[
    bool,
    typer.Option(
        "--json",
        help="Emit structured JSON to stdout instead of human-readable output (useful for scripting / agents)",
    ),
]



"""rule.py — FoBiS.py ``rule`` subcommand."""

import typer

from ._app import _ns, app
from ._constants import __extensions_inc__, __extensions_parsed__
from ._options import *


@app.command("rule")
def cmd_rule(
    ctx: typer.Context,
    # fobos group
    fobos: FobosOpt = None,
    fobos_case_insensitive: FciOpt = False,
    mode: ModeOpt = None,
    lmodes: LmodesOpt = False,
    print_fobos_template: PrintFobosTemplateOpt = False,
    # compiler group
    compiler: CompilerOpt = "gnu",
    fc: FcOpt = None,
    cflags: CflagsOpt = None,
    lflags: LflagsOpt = None,
    modsw: ModswOpt = None,
    mpi: MpiOpt = False,
    openmp: OpenmpOpt = False,
    openmp_offload: OpenmpOffloadOpt = False,
    coarray: CoarrayOpt = False,
    coverage: CoverageOpt = False,
    profile: ProfileOpt = False,
    mklib: MklibOpt = None,
    ar: ArOpt = "ar",
    arflags: ArflagsOpt = "-rcs",
    ranlib: RanlibOpt = "ranlib",
    cflags_heritage: CflagsHeritageOpt = False,
    track_build: TrackBuildOpt = False,
    # directories group
    src: SrcOpt = None,
    build_dir: BuildDirOpt = "./",
    obj_dir: ObjDirOpt = "./obj/",
    mod_dir: ModDirOpt = "./mod/",
    lib_dir: LibDirOpt = None,
    include: IncludeListOpt = None,
    exclude_dirs: ExcludeDirsOpt = None,
    disable_recursive_search: DisableRecursiveSearchOpt = False,
    # files group
    target: TargetOpt = None,
    output: OutputOpt = None,
    exclude: ExcludeOpt = None,
    libs: LibsOpt = None,
    vlibs: VlibsOpt = None,
    ext_libs: ExtLibsOpt = None,
    ext_vlibs: ExtVlibsOpt = None,
    dependon: DependonOpt = None,
    inc: IncOpt = None,
    extensions: ExtensionsOpt = None,
    build_all: BuildAllOpt = False,
    # preprocessor group
    preprocessor: PreprocessorOpt = None,
    preproc: PreprocOpt = None,
    preprocessor_args: PreprocessorArgsOpt = "",
    preprocessor_no_o: PreprocessorNoOOpt = False,
    preprocessor_dir: PreprocessorDirOpt = None,
    preprocessor_ext: PreprocessorExtOpt = None,
    # rules group
    execute: ExecuteOpt = None,
    list_rules: ListRulesOpt = False,
    # intrinsic rules group
    get: GetOpt = None,
    get_output_name: GetOutputNameOpt = False,
    ford: FordOpt = None,
    gcov_analyzer: GcovAnalyzerOpt = None,
    is_ascii_kind_supported: IsAsciiKindSupportedOpt = False,
    is_ucs4_kind_supported: IsUcs4KindSupportedOpt = False,
    is_float128_kind_supported: IsFloat128KindSupportedOpt = False,
    # fancy group
    force_compile: ForceCompileOpt = False,
    colors: ColorsOpt = False,
    log: LogOpt = False,
    graph: GraphOpt = False,
    quiet: QuietOpt = False,
    verbose: VerboseOpt = False,
    jobs: JobsOpt = 1,
    makefile: MakefileOpt = None,
):
    """Execute special rules or user-defined ones from a fobos file."""
    ctx.ensure_object(dict)
    ctx.obj["cliargs"] = _ns(
        which="rule",
        fobos=fobos,
        fobos_case_insensitive=fobos_case_insensitive,
        mode=mode,
        lmodes=lmodes,
        print_fobos_template=print_fobos_template,
        compiler=compiler.lower(),
        fc=fc,
        cflags=cflags,
        lflags=lflags,
        modsw=modsw,
        mpi=mpi,
        openmp=openmp,
        openmp_offload=openmp_offload,
        coarray=coarray,
        coverage=coverage,
        profile=profile,
        mklib=mklib,
        ar=ar,
        arflags=arflags,
        ranlib=ranlib,
        cflags_heritage=cflags_heritage,
        track_build=track_build,
        src=src or ["./"],
        build_dir=build_dir,
        obj_dir=obj_dir,
        mod_dir=mod_dir,
        lib_dir=lib_dir or [],
        include=include or [],
        exclude_dirs=exclude_dirs or [],
        disable_recursive_search=disable_recursive_search,
        target=target,
        output=output,
        exclude=exclude or [],
        libs=libs or [],
        vlibs=vlibs or [],
        ext_libs=ext_libs or [],
        ext_vlibs=ext_vlibs or [],
        dependon=dependon or [],
        inc=inc or list(__extensions_inc__),
        extensions=extensions or list(__extensions_parsed__),
        build_all=build_all,
        preprocessor=preprocessor,
        preproc=preproc,
        preprocessor_args=preprocessor_args,
        preprocessor_no_o=preprocessor_no_o,
        preprocessor_dir=preprocessor_dir,
        preprocessor_ext=preprocessor_ext or [],
        execute=execute,
        list=list_rules,
        get=get,
        get_output_name=get_output_name,
        ford=ford,
        gcov_analyzer=gcov_analyzer,
        is_ascii_kind_supported=is_ascii_kind_supported,
        is_ucs4_kind_supported=is_ucs4_kind_supported,
        is_float128_kind_supported=is_float128_kind_supported,
        force_compile=force_compile,
        colors=colors,
        log=log,
        graph=graph,
        quiet=quiet,
        verbose=verbose,
        jobs=jobs,
        makefile=makefile,
    )



"""install.py — FoBiS.py ``install`` subcommand."""

from typing import Annotated

import typer

from ._app import _ns, app
from ._options import *


@app.command("install")
def cmd_install(
    ctx: typer.Context,
    repo: Annotated[
        str | None, typer.Argument(help='GitHub repository: "user/repo" shorthand or full HTTPS URL')
    ] = None,
    # fobos group
    fobos: FobosOpt = None,
    fobos_case_insensitive: FciOpt = False,
    mode: ModeOpt = None,
    lmodes: LmodesOpt = False,
    print_fobos_template: PrintFobosTemplateOpt = False,
    # directories group (install variant)
    build_dir: BuildDirOpt = "./",
    prefix: Annotated[
        str, typer.Option("--prefix", "-p", help="Prefix path where built objects are installed [default: ./]")
    ] = "./",
    bin: Annotated[str, typer.Option("--bin", help="Sub-directory for executables [default: bin/]")] = "bin/",
    lib: Annotated[str, typer.Option("--lib", help="Sub-directory for libraries [default: lib/]")] = "lib/",
    include: Annotated[
        str, typer.Option("--include", help="Sub-directory for include files [default: include/]")
    ] = "include/",
    # GitHub install options
    branch: Annotated[
        str | None, typer.Option("--branch", help="Branch to check out when installing from GitHub")
    ] = None,
    tag: Annotated[str | None, typer.Option("--tag", help="Tag to check out when installing from GitHub")] = None,
    rev: Annotated[
        str | None, typer.Option("--rev", help="Commit revision to check out when installing from GitHub")
    ] = None,
    update: Annotated[
        bool, typer.Option("--update", help="Re-fetch (git pull) before building and installing")
    ] = False,
    no_build: Annotated[bool, typer.Option("--no-build", help="Clone only — skip building and installing")] = False,
    deps_dir: Annotated[
        str | None, typer.Option("--deps-dir", help="Directory for cloning GitHub repositories [default: ~/.fobis/]")
    ] = None,
    # fancy group
    force_compile: ForceCompileOpt = False,
    colors: ColorsOpt = False,
    log: LogOpt = False,
    graph: GraphOpt = False,
    quiet: QuietOpt = False,
    verbose: VerboseOpt = False,
    jobs: JobsOpt = 1,
    makefile: MakefileOpt = None,
):
    """Install previously built files, or clone+build+install a GitHub-hosted FoBiS project."""
    ctx.ensure_object(dict)
    ctx.obj["cliargs"] = _ns(
        which="install",
        repo=repo,
        fobos=fobos,
        fobos_case_insensitive=fobos_case_insensitive,
        mode=mode,
        lmodes=lmodes,
        print_fobos_template=print_fobos_template,
        build_dir=build_dir,
        prefix=prefix,
        bin=bin,
        lib=lib,
        include=include,
        branch=branch,
        tag=tag,
        rev=rev,
        update=update,
        no_build=no_build,
        deps_dir=deps_dir,
        force_compile=force_compile,
        colors=colors,
        log=log,
        graph=graph,
        quiet=quiet,
        verbose=verbose,
        jobs=jobs,
        makefile=makefile,
    )



"""
_completions.py — Typer autocompletion callbacks for FoBiS.py CLI.
"""

# Copyright (C) 2015  Stefano Zaghi
#
# This file is part of FoBiS.py.
#
# FoBiS.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FoBiS.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FoBiS.py. If not, see <http://www.gnu.org/licenses/>.

import typer

from ._constants import __compiler_supported__, __extensions_parsed__


def _complete_compiler(incomplete: str):
    return [c for c in __compiler_supported__ if c.startswith(incomplete.lower())]


def _complete_mklib(incomplete: str):
    return [m for m in ("static", "shared") if m.startswith(incomplete)]


def _complete_extensions(incomplete: str):
    return [e for e in __extensions_parsed__ if e.startswith(incomplete)]


def _complete_doctests_preprocessor(incomplete: str):
    return [p for p in ("cpp", "fpp") if p.startswith(incomplete)]


def _complete_fobos_mode(ctx: typer.Context, incomplete: str):
    import configparser
    import os

    fobos_path = ctx.params.get("fobos") or "fobos"
    if not os.path.exists(fobos_path):
        return []
    cp = configparser.RawConfigParser()
    cp.read(fobos_path)
    if cp.has_option("modes", "modes"):
        modes = [m.strip() for m in cp.get("modes", "modes").split()]
        return [m for m in modes if m.startswith(incomplete)]
    return [
        s for s in cp.sections() if s.startswith(incomplete) and s not in ("modes", "rules", "dependencies", "project")
    ]
